#include <assert.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "group.h"
#include "ray.h"
#include "line.h"
#include "sphere.h"
#include "cone.h"
#include "box.h"
#include "cylinder.h"
#include "triangle.h"
#include "bmp.h"
#include "implemented.h"

/****
 *
 * The meat of the ray tracer is in this file.
 * You should implement your controlling code
 * in the RayTrace function, and by adding  
 * new functions to this file.
 * The other significant ray tracing code
 * is in the primitive .c files (box, cone, etc.) 
 *
 ****/


/****
 * 
 * The globals are all set by the parser
 * If you distate global variables,
 * you might move some of these into 
 * RayTrace()
 *
 ****/


/****
 * A call to this function invokes the ray tracer. It returns 
 * a pointer to the image when it's done.
 ***/

Image* RayTrace(const char *fileName,int width,int height,
		int rLimit, float cLimit){
  Scene scene;
  Image *img;

  /* For the parsing we turn of the implemented flag */
  implementedFlag=0;

  /* The image width/height determines the aspect ratio
   * This ratio times the height angle gives the width angle 
   * of the viewing volulme. */

  scene.camera.aspectRatio = (Flt)width/(Flt)height;

  img = ImageNew(width,height);

  /* This parses the entire scene graph into a node tree structure containing
   * the primitive shaped and the transform hierarchy at each node
   * The parser also mallocs lottsa space for materials, lights, and textures.
   * The camera and environment structures are also filled.*/
  scene.read(fileName);

  /* After parsing we turn the implemented flag on to announce when an
   * unimplemented method is being invoked. */
  implementedFlag=1;

  /* A debugging tool. This function prints
   * and indented tree of the scene graph
   * to the scree. Coordinates and matrices
   * are given.*/
  scene.shape->write(0);


  /****
   *
   * Do ray tracing here!!!
   *
   ****/

  /* Clean up is taken care of for you :') */
  scene.free();
  return img;
}

/*//////////////////////////////////////*/
Point3D GetColor (
  Scene scene, 
  Ray ray,
  IntersectionInfo iInfo,
  int rDepth,
  float cLimit
) {
  /* WORK HERE */

  return Point3D (1.0,1.0,1.0); /* for now, return white */ 
}
